;-------------------------------------------------------------------------------
; GoatTracker V2.0 gamemusic playroutine(optimized for same gatetimer/firstwave)
;
; NOTE: This playroutine source code does not fall under the GPL license!
; Use it freely for any purpose, commercial or noncommercial.
;-------------------------------------------------------------------------------

                processor 6502
                org $1000

        ; "Virtual addresses" for relocation

mt_insad        = $4001
mt_inssr        = $4101
mt_inswaveptr   = $4201
mt_inspulseptr  = $4301
mt_insfilterptr = $4401
mt_insvibdelay  = $4501
mt_insvibparam  = $4601
mt_wavetbl      = $4901
mt_notetbl      = $4a02
mt_pulsetimetbl = $4b01
mt_pulsespdtbl  = $4c01
mt_filtertimetbl = $4d01
mt_filterspdtbl = $4e01
mt_songtbllo    = $4f00
mt_songtblhi    = $5000
mt_patttbllo    = $5100
mt_patttblhi    = $5200

        ; Used zeropage temp variables

mt_temp1        = $fc
mt_temp2        = $fd

        ; Defines for the music data
        ; Patterndata notes

ENDPATT         = $00
INS             = $00
FX              = $40
FXONLY          = $50
NOTE            = $60
REST            = $bd
KEYOFF          = $be
KEYON           = $bf
FIRSTPACKEDREST = $c0
PACKEDREST      = $00

        ;Effects

DONOTHING    = $00
PORTAUP      = $01
PORTADOWN    = $02
TONEPORTA    = $03
VIBRATO      = $04
SETAD        = $05
SETSR        = $06
SETWAVE      = $07
SETWAVEPTR   = $08
SETPULSEPTR  = $09
SETFILTPTR   = $0a
SETFILTCTRL  = $0b           ;Resonance/channels
SETFILTCUTOFF = $0c
SETMASTERVOL = $0d
SETFUNKTEMPO = $0e
SETTEMPO     = $0f

        ; Orderlist commands

REPEAT          = $d0
TRANSDOWN       = $e0
TRANS           = $f0
TRANSUP         = $f0
LOOPSONG        = $ff

        ;Wave,pulse,filtertable comands

LOOPWAVE        = $ff
LOOPPULSE       = $ff
LOOPFILTER      = $ff
SETPULSE        = $80
SETFILTER       = $80
SETCUTOFF       = $00

        ;Jump table

                jmp mt_init
                jmp mt_play
                jmp mt_playsfx
                jmp mt_setmastervolume

        ;Playroutine

mt_play:        ldx #$00                        ;Channel index

        ;Song initialization

mt_initsongnum: ldy #$00
                bmi mt_filterstep
                txa
                ldx #41
mt_resetloop:   sta mt_chnsongptr,x             ;Reset sequencer + voice
                dex                             ;variables on all channels
                bpl mt_resetloop
                sta $d415                       ;Reset filter cutoff lowbyte
                sta mt_filterctrl+1             ;Switch filter off & reset
                sta mt_filterstep+1             ;step-programming
                stx mt_initsongnum+1            ;Reset initflag
                tax
                jsr mt_initchn
                ldx #$07
                jsr mt_initchn
                ldx #$0e
mt_initchn:     tya
                iny
                sta mt_chnsongnum,x             ;Store index to songtable
mt_defaulttempo:dc.b $ff,$00                    ;Set default tempo
                sta mt_chntempo,x
                lda #$01
                sta mt_chncounter,x             ;Reset counter
                sta mt_chninstr,x               ;Reset instrument
                jmp mt_loadregs                 ;Load waveform

        ;Filter execution

mt_filterstep:  ldy #$00                        ;See if filter stopped
                beq mt_filterdone
mt_filtertime:  lda #$00                        ;See if time left for mod.
                bne mt_filtermod                ;step
mt_newfilterstep:
                lda mt_filtertimetbl-1,y        ;$80-> = set filter parameters
                beq mt_setcutoff                ;$00 = set cutoff
                bpl mt_newfiltermod
mt_setfilter:   asl                             ;Set passband
                sta mt_filtertype+1
                lda mt_filterspdtbl-1,y         ;Set resonance/channel
                sta mt_filterctrl+1
                lda mt_filtertimetbl,y          ;Check for cutoff setting
                bne mt_nextfilterstep2          ;following immediately
mt_setcutoff2:  iny
mt_setcutoff:   lda mt_filterspdtbl-1,y         ;Take cutoff value
                sta mt_filtercutoff+1
                jmp mt_nextfilterstep
mt_newfiltermod:sta mt_filtertime+1             ;$01-$7f = new modulation step
mt_filtermod:   lda mt_filterspdtbl-1,y         ;Take filter speed
                clc
                adc mt_filtercutoff+1
                sta mt_filtercutoff+1
                dec mt_filtertime+1
                bne mt_storecutoff
mt_nextfilterstep:
                lda mt_filtertimetbl,y           ;Jump in filtertable?
mt_nextfilterstep2:
                cmp #LOOPFILTER
                iny
                tya
                bcc mt_nofilterjump
                lda mt_filterspdtbl-1,y          ;Take jump point
mt_nofilterjump:sta mt_filterstep+1
mt_filterdone:
mt_filtercutoff:lda #$00
mt_storecutoff: sta $d416
mt_filterctrl:  lda #$00
                sta $d417
mt_filtertype:  lda #$00
mt_masterfader: ora #$0f                        ;Master volume fader
                sta $d418

                jsr mt_execchn
                ldx #$07
                jsr mt_execchn
                ldx #$0e

        ;Channel execution

mt_execchn:     dec mt_chncounter,x               ;See if tick 0
                beq mt_tick0

        ;Ticks 1-n

mt_notick0:     bpl mt_effects
                lda mt_chntempo,x               ;Reload tempo if negative
                cmp #$02
                bcs mt_nofunktempo              ;Funktempo: bounce between
                tay                             ;funktable indexes 0,1
                eor #$01
                sta mt_chntempo,x
                lda mt_funktempotbl,y
mt_nofunktempo: sta mt_chncounter,x
mt_effects:     jmp mt_waveexec

        ;Tick 0

mt_tick0:       ldy mt_chnnewfx,x               ;Setup tick 0 FX jumps
                lda mt_tick0jumptbl,y
                sta mt_tick0jump1+1
                sta mt_tick0jump2+1

        ;Sequencer advance

mt_checknewpatt:lda mt_chnpattptr,x             ;Fetch next pattern?
                bne mt_nonewpatt
mt_sequencer:   ldy mt_chnsongnum,x
                lda mt_songtbllo,y              ;Get address of sequence
                sta mt_temp1
                lda mt_songtblhi,y
                sta mt_temp2
                ldy mt_chnsongptr,x
                lda (mt_temp1),y                ;Get pattern from sequence
                cmp #LOOPSONG                   ;Check for loop
                bcc mt_noloop
                iny
                lda (mt_temp1),y
                tay
                lda (mt_temp1),y
mt_noloop:      cmp #TRANSDOWN                  ;Check for transpose
                bcc mt_notrans
                sbc #TRANS
                sta mt_chntrans,x
                iny
                lda (mt_temp1),y
mt_notrans:     cmp #REPEAT                     ;Check for repeat
                bcc mt_pattok
                sbc #REPEAT
                sta mt_chnrepeat,x
                iny
                lda (mt_temp1),y
mt_pattok:      sta mt_chnpattnum,x             ;Store pattern number
                lda mt_chnrepeat,x              ;Check for repeat here
                beq mt_norepeat
                dec mt_chnrepeat,x              ;Decrement repeat
                bpl mt_sequencerdone
mt_norepeat:    iny
mt_sequencerdone:
                tya
                sta mt_chnsongptr,x             ;Store songposition

        ;New note start

mt_nonewpatt:   ldy mt_chninstr,x
                lda mt_chnnewnote,x             ;Test new note init flag
                beq mt_nonewnoteinit
mt_newnoteinit: sec
                sbc #NOTE
                sta mt_chnnote,x
                lda #$00
                sta mt_chnfx,x                  ;Reset effect
                sta mt_chnnewnote,x             ;Reset newnote action
                lda mt_insvibdelay-1,y          ;Load instrument vibrato
                sta mt_chnvibdelay,x
                lda mt_chnnewfx,x               ;If toneportamento, skip
                cmp #TONEPORTA                  ;most of note init
                beq mt_nonewnoteinit
                lda mt_inspulseptr-1,y          ;Load pulseptr (if nonzero)
                beq mt_skippulse
                sta mt_chnpulseptr,x
                lda #$00                        ;Reset pulse step duration
                sta mt_chnpulsetime,x
mt_skippulse:   lda mt_insfilterptr-1,y         ;Load filterptr (if nonzero)
                beq mt_skipfilter
                sta mt_filterstep+1
                lda #$00
                sta mt_filtertime+1
mt_skipfilter:  lda mt_insad-1,y                ;Load Attack/Decay
                sta mt_chnad,x
                lda mt_inssr-1,y
                sta mt_chnsr,x
mt_firstwave:   dc.b $ff,$00                    ;Load first frame waveform
                sta mt_chnwave,x
                lda mt_inswaveptr-1,y           ;Load waveptr
                sta mt_chnwaveptr,x
                lda #$ff
                sta mt_chngate,x                ;Reset gateflag
                lda mt_chnnewparam,x            ;Execute tick 0 FX after
mt_tick0jump1:  jsr mt_tick0_0                  ;newnote init
                jmp mt_loadregs

        ;Tick 0 effect execution

mt_nonewnoteinit:
                lda mt_chnnewparam,x            ;No new note init: exec tick 0
mt_tick0jump2:  jsr mt_tick0_0                  ;FX, and wavetable afterwards

        ;Wavetable execution

mt_waveexec:    ldy mt_chnwaveptr,x
                beq mt_wavedone
                lda mt_wavetbl-1,y
                cmp #$08                        ;0-7 used as delay
                bcs mt_nowavedelay              ;+ no wave change
                cmp mt_chnwavetime,x
                beq mt_nowavechange
                inc mt_chnwavetime,x
                bne mt_wavedone
mt_nowavedelay: sta mt_chnwave,x
mt_nowavechange:lda mt_wavetbl,y
                cmp #LOOPWAVE                  ;Check for wavetable jump
                iny
                tya
                bcc mt_nowavejump
                clc
                lda mt_notetbl-1,y
mt_nowavejump:  sta mt_chnwaveptr,x
                lda #$00
                sta mt_chnwavetime,x
                lda mt_notetbl-2,y
                bne mt_wavefreq                 ;No frequency-change?

        ;No frequency-change / continuous effect execution

mt_wavedone:    lda mt_chncounter,x             ;No continuous effects on tick0
                beq mt_gatetimer
                lda #$00
                sta mt_temp2                    ;Clear speed highbyte
                ldy mt_chnfx,x
                lda mt_effectjumptbl,y
                sta mt_effectjump+1
                lda mt_chnparam,x
mt_effectjump:  jmp mt_effect_0

        ;Setting note frequency

mt_wavefreq:    bpl mt_wavenoteabs                
                adc mt_chnnote,x
                and #$7f
mt_wavenoteabs: tay
mt_wavenote:    lda mt_freqtbllo,y
                sta mt_chnfreqlo,x
                lda mt_freqtblhi,y
                sta mt_chnfreqhi,x
                lda #$00                        ;Reset vibrato phase
                sta mt_chnvibtime,x

        ;Check for new note fetch

mt_done:        lda mt_chncounter,x             ;Check for gateoff timer
mt_gatetimer:   dc.b $ff,$00
                beq mt_getnewnote               ;Fetch new notes if equal

        ;Pulse execution

mt_pulseexec:   ldy mt_chnpulseptr,x            ;See if pulse stopped
                beq mt_pulseskip
                ora mt_chnpattptr,x             ;Skip when sequencer executed
                beq mt_pulseskip
                lda mt_chnpulsetime,x           ;Pulse step counter time left?
                bne mt_pulsemod
mt_newpulsestep:lda mt_pulsetimetbl-1,y         ;Set pulse, or new modulation
                bpl mt_newpulsemod              ;step?
mt_setpulse:    sta mt_chnpulsehi,x             ;Highbyte
                lda mt_pulsespdtbl-1,y          ;Lowbyte
                sta mt_chnpulselo,x
                jmp mt_nextpulsestep
mt_newpulsemod: sta mt_chnpulsetime,x
mt_pulsemod:    lda mt_pulsespdtbl-1,y          ;Take pulse speed
                asl
                bcc mt_pulseup
                clc
                dec mt_chnpulsehi,x
mt_pulseup:     adc mt_chnpulselo,x             ;Add pulse lowbyte
                sta mt_chnpulselo,x
                bcc mt_pulsenotover
                inc mt_chnpulsehi,x
mt_pulsenotover:dec mt_chnpulsetime,x
                bne mt_pulsedone
mt_nextpulsestep:
                lda mt_pulsetimetbl,y           ;Jump in pulsetable?
                cmp #LOOPPULSE
                iny
                tya
                bcc mt_nopulsejump
                lda mt_pulsespdtbl-1,y          ;Take jump point
mt_nopulsejump: sta mt_chnpulseptr,x
mt_pulsedone:
mt_pulseskip:   jmp mt_loadregs

        ;New note fetch

mt_getnewnote:  ldy mt_chnpattnum,x
                lda mt_patttbllo,y
                sta mt_temp1
                lda mt_patttblhi,y
                sta mt_temp2
                ldy mt_chnpattptr,x
                lda (mt_temp1),y
                iny
                cmp #FX
                bcc mt_instr                    ;Instr. change
                cmp #NOTE
                bcc mt_fx                       ;FX
                cmp #FIRSTPACKEDREST
                bcc mt_note                     ;Note only

        ;Packed rest handling

mt_packedrest:  ldy mt_chnpackedrest,x
                bne mt_packedrestnonew
                sta mt_chnpackedrest,x
mt_packedrestnonew:
                inc mt_chnpackedrest,x
                bne mt_loadregs
                inc mt_chnpattptr,x
                bne mt_loadregs

        ;Instrument change

mt_instr:       sta mt_chninstr,x               ;Instrument change, followed
                lda (mt_temp1),y                ;by either FX or note
                iny
                cmp #NOTE
                bcs mt_note

        ;Effect change

mt_fx:          cmp #FXONLY                     ;Note follows?
                and #$0f
                sta mt_chnnewfx,x
                beq mt_fx_noparam               ;Effect 0 - no param.
                lda (mt_temp1),y
                iny
                sta mt_chnnewparam,x
mt_fx_noparam:  bcs mt_rest
mt_fx_getnote:  lda (mt_temp1),y
                iny

        ;Note handling

mt_note:        cmp #REST                   ;Rest or gateoff/on?
                bcc mt_normalnote
                beq mt_rest
mt_gate:        ora #$f0
                bne mt_storegate

        ;Prepare for note start; perform hardrestart

mt_normalnote:  adc mt_chntrans,x
                sta mt_chnnewnote,x
                lda mt_chnnewfx,x           ;If toneportamento, no gateoff
                cmp #TONEPORTA
                beq mt_rest
mt_adparam:     dc.b $ff,$00                ;Hard restart
                sta mt_chnad,x
mt_srparam:     dc.b $ff,$00
                sta mt_chnsr,x
                lda #$fe
mt_storegate:   sta mt_chngate,x

        ;Check for end of pattern

mt_rest:        lda (mt_temp1),y
                beq mt_endpatt
                tya
mt_endpatt:     sta mt_chnpattptr,x

        ;Load voice registers

mt_loadregs:    lda mt_chnsfx,x
                bne mt_sfxexec
                lda mt_chnad,x
                sta $d405,x
                lda mt_chnsr,x
                sta $d406,x
                lda mt_chnpulselo,x
                sta $d402,x
                lda mt_chnpulsehi,x
                sta $d403,x
mt_loadregs2:   lda mt_chnfreqlo,x
                sta $d400,x
                lda mt_chnfreqhi,x
                sta $d401,x
                lda mt_chnwave,x
                and mt_chngate,x
                sta $d404,x
                rts

        ;Sound FX code

mt_sfxexec:     tay
                lda mt_chnsfxlo,x
                sta mt_temp1
                lda mt_chnsfxhi,x
                sta mt_temp2
                lda #$fe
                sta mt_chngate,x
                lda #$00
                sta mt_chnwaveptr,x
                inc mt_chnsfx,x
                cpy #$02
                beq mt_sfxexec_frame0
                bcs mt_sfxexec_framen
                sta $d405,x                ;Hardrestart before sound FX
                sta $d406,x                ;begins
                bcc mt_loadregs2

mt_sfxexec_frame0:
                tay
                lda (mt_temp1),y           ;Load ADSR
                sta $d405,x
                iny
                lda (mt_temp1),y
                sta $d406,x
                iny
                lda (mt_temp1),y           ;Load pulse
                sta $d402,x
                sta $d403,x
                lda #$09                   ;Testbit
mt_sfxexec_wavechg:
                sta mt_chnwave,x
                sta $d404,x
mt_sfxexec_done:rts

mt_sfxexec_framen:
                lda (mt_temp1),y
                bne mt_sfxexec_noend
mt_sfxexec_end: sta mt_chnsfx,x
                beq mt_sfxexec_wavechg
mt_sfxexec_noend:
                tay
                lda mt_freqtbllo-$80,y        ;Get frequency
                sta $d400,x
                lda mt_freqtblhi-$80,y
                sta $d401,x
                ldy mt_chnsfx,x
                lda (mt_temp1),y              ;Then take a look at the next
                beq mt_sfxexec_done           ;byte
                cmp #$82                      ;Is it a waveform or a note?
                bcs mt_sfxexec_done
                inc mt_chnsfx,x
                bcc mt_sfxexec_wavechg

        ;Tick 0 effect code

        ;0 Instrument vibrato

mt_tick0_0:     lda mt_insvibparam-1,y
                jmp mt_tick0_34

        ;1,2 Portamentos

mt_tick0_12:    tay
                lda #$00
                sta mt_chnvibtime,x
                tya

        ;3,4 Toneportamento, Vibrato

mt_tick0_34:    sta mt_chnparam,x
                lda mt_chnnewfx,x
                sta mt_chnfx,x
                rts

        ;5 Set AD

mt_tick0_5:     sta mt_chnad,x
                rts

        ;6 Set Sustain/Release

mt_tick0_6:     sta mt_chnsr,x
                rts

        ;7 Set waveform

mt_tick0_7:     sta mt_chnwave,x
                rts

        ;8 Set wavepointer

mt_tick0_8:     sta mt_chnwaveptr,x
                lda #$00                        ;Make sure possible delayed
                sta mt_chnwavetime,x            ;waveform execution goes
                rts                             ;correctly

        ;9 Set pulsepointer

mt_tick0_9:     sta mt_chnpulseptr,x
                lda #$00                        ;Reset pulse step duration
                sta mt_chnpulsetime,x
                rts

        ;a Set filterpointer

mt_tick0_a:     ldy #$00
                sty mt_filtertime+1
mt_tick0_a_step:sta mt_filterstep+1
                rts

        ;b Set filtercontrol (channels & resonance)

mt_tick0_b:     sta mt_filterctrl+1
                beq mt_tick0_a_step          ;If 0, stop also step-programming
                rts

        ;c Set cutoff

mt_tick0_c:     sta mt_filtercutoff+1
                rts

        ;d Set mastervolume

mt_setmastervolume:
mt_tick0_d:     sta mt_masterfader+1
                rts

        ;e Funktempo

mt_tick0_e:     tay
                lsr
                lsr
                lsr
                lsr
                sta mt_funktempotbl
                tya
                and #$0f
                sta mt_funktempotbl+1
                lda #$00
                beq mt_tick0_f_setglobaltempo

        ;f Set Tempo

mt_tick0_f:     bmi mt_tick0_f_setchantempo     ;Channel or global tempo?
mt_tick0_f_setglobaltempo:
                sta mt_chntempo
                sta mt_chntempo+7
                sta mt_chntempo+14
                rts
mt_tick0_f_setchantempo:
                and #$7f
                sta mt_chntempo,x
                rts

        ;Check for page cross error (only lowbyte of jump modified)

                if (mt_tick0_0 & $ff00) != (mt_tick0_f & $ff00)
                err
                endif

        ;Init routine

mt_init:        sta mt_initadc+1
                asl
mt_initadc:     adc #$00
                sta mt_initsongnum+1
                rts

        ;Sound FX init routine

mt_playsfx:     sta mt_playsfxlo+1
                sty mt_playsfxhi+1
                lda mt_chnsfx,x                   ;Need a priority check?
                beq mt_playsfxok
                tya                               ;Check address highbyte
                cmp mt_chnsfxhi,x
                bcc mt_playsfxskip                ;Lower than current -> skip
                bne mt_playsfxok                  ;Higher than current -> OK
                lda mt_playsfxlo+1                ;Check address lowbyte
                cmp mt_chnsfxlo,x
                bcc mt_playsfxskip                ;Lower than current -> skip
mt_playsfxok:   lda #$01
                sta mt_chnsfx,x
mt_playsfxlo:   lda #$00
                sta mt_chnsfxlo,x
mt_playsfxhi:   lda #$00
                sta mt_chnsfxhi,x
mt_playsfxskip: rts

        ;Continuous effect code

        ;0 Instrument vibrato

mt_effect_0:    ldy mt_chnvibdelay,x
                beq mt_effect_0_donothing         ;Delay 0 = no vibrato at all
                cpy #$01
                beq mt_effect_4
                dec mt_chnvibdelay,x
mt_effect_0_donothing:
                jmp mt_done

        ;1 Portamento up

mt_effect_1:    jsr mt_makespeed
                bcc mt_freqadd

        ;2 Portamento down

mt_effect_2:    jsr mt_makespeed
                sec
                bcs mt_freqsub

        ;4 Vibrato

mt_effect_4:    tay
                and #$f0                ;Clear low bits of speed
                sta mt_temp1
                tya
                and #$0f
                sta mt_effect_4_cmp+1
                lda mt_chnvibtime,x
                bmi mt_effect_4_nodir
mt_effect_4_cmp:cmp #$00
                bcc mt_effect_4_nodir2
                beq mt_effect_4_nodir
                eor #$ff
mt_effect_4_nodir:
                clc
mt_effect_4_nodir2:
                adc #$02
mt_vibdone:     sta mt_chnvibtime,x
                lsr
                bcc mt_freqadd
                bcs mt_freqsub

        ;3 Toneportamento

mt_effect_3:    ldy mt_chnnote,x
                cmp #$00
                beq mt_effect_3_found          ;Speed $00 = tie note
                jsr mt_makespeed
                lda mt_freqtbllo,y              ;Calculate offset to the
                sec                             ;right frequency
                sbc mt_chnfreqlo,x
                sta mt_effect_3_templo+1
                lda mt_freqtblhi,y
                sbc mt_chnfreqhi,x
                sta mt_effect_3_temphi+1
                bmi mt_effect_3_down            ;If negative, have to go down

mt_effect_3_up: lda mt_temp2                    ;Now compare toneportamento
                cmp mt_effect_3_temphi+1         ;speed to the offset
                bne mt_effect_3_up_nolowbyte
                lda mt_temp1
                cmp mt_effect_3_templo+1
mt_effect_3_up_nolowbyte:
                bcs mt_effect_3_found           ;If speed greater, we're done
mt_freqadd:     lda mt_chnfreqlo,x
                adc mt_temp1
                sta mt_chnfreqlo,x
                lda mt_chnfreqhi,x
                adc mt_temp2
                sta mt_chnfreqhi,x
                jmp mt_done

mt_effect_3_down:
mt_effect_3_templo:
                lda #$00                        ;Add speed to the negative
                adc mt_temp1                    ;offset
mt_effect_3_temphi:
                lda #$00
                adc mt_temp2
                bpl mt_effect_3_found           ;If goes positive, we're done
                sec
mt_freqsub:     lda mt_chnfreqlo,x
                sbc mt_temp1
                sta mt_chnfreqlo,x
                lda mt_chnfreqhi,x
                sbc mt_temp2
                sta mt_chnfreqhi,x
                jmp mt_done

mt_effect_3_found:
                jmp mt_wavenote

mt_makespeed:   asl
                rol mt_temp2
                asl
                rol mt_temp2
                sta mt_temp1
                rts

        ;Check for page cross error (only lowbyte of jump modified)

                if (mt_effect_0 & $ff00) != (mt_effect_3 & $ff00)
                err
                endif

mt_chnsongptr:    dc.b 0
mt_chntrans:      dc.b 0
mt_chnrepeat:     dc.b 0
mt_chnpattptr:    dc.b 0
mt_chnpackedrest: dc.b 0
mt_chnnewfx:      dc.b 0
mt_chnnewparam:   dc.b 0

                dc.b 0,0,0,0,0,0,0
                dc.b 0,0,0,0,0,0,0

mt_chnfx:         dc.b 0
mt_chnparam:      dc.b 0
mt_chnnewnote:    dc.b 0
mt_chnwaveptr:    dc.b 0
mt_chnwave:       dc.b 0
mt_chnpulseptr:   dc.b 0
mt_chnpulsetime:  dc.b 0

                dc.b 0,0,0,0,0,0,0
                dc.b 0,0,0,0,0,0,0

mt_chnsongnum:    dc.b 0
mt_chnpattnum:    dc.b 0
mt_chnnote:       dc.b 0
mt_chnfreqlo:     dc.b 0
mt_chnfreqhi:     dc.b 0
mt_chnpulselo:    dc.b 0
mt_chnpulsehi:    dc.b 0

                dc.b 0,0,0,0,0,0,0
                dc.b 0,0,0,0,0,0,0

mt_chninstr:      dc.b 1
mt_chntempo:      dc.b 0
mt_chncounter:    dc.b 0
mt_chnvibtime:    dc.b 0
mt_chnvibdelay:   dc.b 0
mt_chnwavetime:   dc.b 0
mt_chngate:       dc.b $fe

                dc.b 1,0,0,0,0,0,$fe
                dc.b 1,0,0,0,0,0,$fe
                
mt_chnad:         dc.b 0
mt_chnsr:         dc.b 0
mt_chnsfx:        dc.b 0
mt_chnsfxlo:      dc.b 0
mt_chnsfxhi:      dc.b 0
mt_chnunused1:    dc.b 0
mt_chnunused2:    dc.b 0

                dc.b 0,0,0,0,0,0,0
                dc.b 0,0,0,0,0,0,0

mt_freqtblhi:   dc.b $01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$02
                dc.b $02,$02,$02,$02,$02,$02,$03,$03,$03,$03,$03,$04
                dc.b $04,$04,$04,$05,$05,$05,$06,$06,$06,$07,$07,$08
                dc.b $08,$09,$09,$0a,$0a,$0b,$0c,$0d,$0d,$0e,$0f,$10
                dc.b $11,$12,$13,$14,$15,$17,$18,$1a,$1b,$1d,$1f,$20
                dc.b $22,$24,$27,$29,$2b,$2e,$31,$34,$37,$3a,$3e,$41
                dc.b $45,$49,$4e,$52,$57,$5c,$62,$68,$6e,$75,$7c,$83
                dc.b $8b,$93,$9c,$a5,$af,$b9,$c4,$d0,$dd,$ea,$f8,$ff

mt_freqtbllo:   dc.b $17,$27,$39,$4b,$5f,$74,$8a,$a1,$ba,$d4,$f0,$0e
                dc.b $2d,$4e,$71,$96,$be,$e8,$14,$43,$74,$a9,$e1,$1c
                dc.b $5a,$9c,$e2,$2d,$7c,$cf,$28,$85,$e8,$52,$c1,$37
                dc.b $b4,$39,$c5,$5a,$f7,$9e,$4f,$0a,$d1,$a3,$82,$6e
                dc.b $68,$71,$8a,$b3,$ee,$3c,$9e,$15,$a2,$46,$04,$dc
                dc.b $d0,$e2,$14,$67,$dd,$79,$3c,$29,$44,$8d,$08,$b8
                dc.b $a1,$c5,$28,$cd,$ba,$f1,$78,$53,$87,$1a,$10,$71
                dc.b $42,$89,$4f,$9b,$74,$e2,$f0,$a6,$0e,$33,$20,$ff

          ;Command tables

mt_tick0jumptbl:dc.b <mt_tick0_0
                dc.b <mt_tick0_12
                dc.b <mt_tick0_12
                dc.b <mt_tick0_34
                dc.b <mt_tick0_34
                dc.b <mt_tick0_5
                dc.b <mt_tick0_6
                dc.b <mt_tick0_7
                dc.b <mt_tick0_8
                dc.b <mt_tick0_9
                dc.b <mt_tick0_a
                dc.b <mt_tick0_b
                dc.b <mt_tick0_c
                dc.b <mt_tick0_d
                dc.b <mt_tick0_e
                dc.b <mt_tick0_f

mt_effectjumptbl:
                dc.b <mt_effect_0
                dc.b <mt_effect_1
                dc.b <mt_effect_2
                dc.b <mt_effect_3
                dc.b <mt_effect_4

mt_funktempotbl:dc.b 8,5

